import _classCallCheck from '@babel/runtime/helpers/classCallCheck';
import _createClass from '@babel/runtime/helpers/createClass';
import ClassList from 'classlist';
import throttle from 'lodash.throttle';

var Adaptive = /*#__PURE__*/function () {
  _createClass(Adaptive, null, [{
    key: "install",

    /**
     * Viewport describing object
     * @typedef viewport
     * @type {object} - viewport size
     * @property {number} width - viewport width
     * @property {number} height - viewport height
     */

    /**
     * Device configuration object
     * @typedef device
     * @type {object}
     * @property {function({width: number, height: number})} [if] - custom device condition callback
     * @property {string|HTMLElement} [element=window] - element selector that will be used for detecting viewport
     * @property {number} [rem] - static rem value, overrides k and base calculation
     * @property {number} [k=1] - Additional coefficient for rem calculation
     * @property {viewport} [base] - base width and height for rem calculation
     * @property {viewport} [from] - minimal width and height for setting device
     * @property {viewport} [to] - maximal width and height for setting device
     * @property {boolean} [setDevice=true] - class, no-class and device set toggle
     */

    /**
     * Plugin configuration object
     * @typedef globalConf
     * @type {object}
     * @property {number} [rem=10] - default rem value in case no breakpoints matched
     * @property {number} [throttle=17] - frequency of viewport data update
     * @property {number} [orientationTestCount=50] - orientationchange event end detection tries
     * @property {number} [orientationChangeTimeout=1000] - maximum orientationchange event duration before viewport update
     */

    /**
     * Vue plugin install function
     * @param Vue - Vue framework object
     * @param {{string: device}} config - adaptive configuration file
     * @param {globalConf} config.global - Plugin's instance-level config
     */
    value: function install(Vue, config) {
      var adaptive = new this(Vue, config);
      Vue.adaptive = adaptive;
      Object.defineProperty(Vue.prototype, "$adaptive", {
        /**
         * Adaptive current state getter
         * @returns {Adaptive} - Adaptive current state observable
         */
        get: function get() {
          return adaptive;
        }
      });
    }
    /**
     * Splitting configuration and creating eventListeners
     * @param Vue - Vue framework object
     * @param {{string: device}} config - adaptive configuration file
     * @param {globalConf} config.global - Plugin's instance-level config
     * @returns {Record<string, >} - reactive model, based on Vue
     */

  }]);

  function Adaptive(Vue, config) {
    _classCallCheck(this, Adaptive);

    // Defaults
    var defaultGlobal = {
      rem: 10,
      throttle: 17,
      orientationTestCount: 25,
      orientationChangeTimeout: 1000
    };
    var defaultConfig = {
      if: null,
      element: window,
      rem: null,
      from: null,
      to: null,
      base: null,
      k: 1,
      setDevice: true
    }; // Global config

    this.globals = Object.assign(defaultGlobal, config.global);
    delete config.global;
    var deviceList = {};

    for (var device in config) {
      if (config.hasOwnProperty(device)) {
        var deviceConfig = config[device] = Object.assign({}, defaultConfig, config[device]);
        device = device.split(":")[0];
        if (deviceConfig.setDevice && !deviceList.hasOwnProperty(device)) deviceList[device] = false;
      }
    }

    this.config = config;
    var data = {
      is: deviceList,
      width: 0,
      height: 0,
      rem: 0
    };
    var vueVersionArray = Vue.version.split(".");
    /**
     * Current adaptive data
     * @type {object}
     * @property {object.<string,boolean>} is - devices status
     * @property {number} width - current viewport width
     * @property {number} height - current viewport height
     * @property {number} rem - current rem size in px
     */

    this.data = vueVersionArray[0] === 2 && vueVersionArray[1] >= 6 ? Vue.observable(data) : new Vue({
      data: data
    }).$data; // Listening to change events

    window.addEventListener("resize", throttle(this.resize.bind(this), this.globals.throttle));
    window.addEventListener("orientationchange", this.orientationChange.bind(this)); // Initializing Adaptive

    this.resize();
    return this.data;
  }
  /**
   * Handle resize of viewport
   */


  _createClass(Adaptive, [{
    key: "resize",
    value: function resize() {
      var viewport = {
        width: window.innerWidth,
        height: window.innerHeight
      };
      var cache = {
        keys: [window],
        values: [viewport]
      };
      var newDeviceList = {};
      var rem = this.globals.rem; // Setting viewport size

      if (this.data.width !== viewport.width) this.data.width = viewport.width;
      if (this.data.height !== viewport.height) this.data.height = viewport.height;

      for (var name in this.config) {
        if (this.config.hasOwnProperty(name)) {
          var device = this.config[name]; // Getting device name

          name = name.split(":")[0]; // Caching elements viewport

          var elementCacheIndex = cache.keys.indexOf(device.element);
          var data = void 0;

          if (elementCacheIndex === -1) {
            var el = device.element instanceof HTMLElement ? device.element : document.querySelector(device.element);
            var clientRect = el.getBoundingClientRect();
            data = {
              width: clientRect.width,
              height: clientRect.height
            };
            cache.keys.push(device.element);
            cache.values.push(data);
          } else data = cache.values[elementCacheIndex]; // Detecting is breakpoints valid
          // Testing if function


          var checked = !(typeof device.if === "function" && !device.if(data) || // Testing min viewport
          device.from && (device.from.width > data.width || device.from.height > data.height) || // Testing max viewport
          device.to && (device.to.width <= data.width || device.to.height <= data.height)); // Testing classes

          if (device.setDevice && !newDeviceList[name]) newDeviceList[name] = checked; // Scale changing

          if (checked) {
            // Setting static rem
            if (device.rem) {
              rem = device.rem; // Setting dynamic rem
            } else if (device.base) {
              var remBases = [];

              if (device.base.width) {
                remBases.push(data.width / device.base.width);
              }

              if (device.base.height) {
                remBases.push(data.height / device.base.height);
              }

              var remBase = Math.min.apply(Math, remBases);
              rem = remBase * device.k * 10;
            }
          }
        }
      }

      this.updateDOM(newDeviceList, rem);
    }
    /**
     * Update documentElement classes and font size
     * @param {object.<string,boolean>} newDeviceList - new devices state
     * @param {number} rem - new rem size calculation
     */

  }, {
    key: "updateDOM",
    value: function updateDOM(newDeviceList, rem) {
      var documentElementClassList = new ClassList(document.documentElement);

      for (var name in newDeviceList) {
        if (newDeviceList.hasOwnProperty(name)) {
          var checked = newDeviceList[name];
          var oldClass = this.data.is[name] ? name : "no-".concat(name);
          var newClass = checked ? name : "no-".concat(name); // Updating classes if changed

          if (oldClass !== newClass || !documentElementClassList.contains(newClass)) {
            if (documentElementClassList.contains(oldClass)) documentElementClassList.remove(oldClass);
            documentElementClassList.add(newClass);
            this.data.is[name] = checked;
          }
        }
      }

      if (rem !== this.data.rem) {
        document.documentElement.style.fontSize = "".concat(rem, "px");
        this.data.rem = rem;
      }
    }
    /**
     * Handle orientationChange event
     */

  }, {
    key: "orientationChange",
    value: function orientationChange() {
      var _this = this;

      var noChangeCount = 0;

      var end = function end() {
        clearInterval(interval);
        clearTimeout(timeout);
        interval = null;
        timeout = null;
      };

      var interval = setInterval(function () {
        var currHeight = window.innerHeight;
        var currWidth = window.innerWidth;

        if (currWidth === _this.data.width && currHeight === _this.data.height) {
          noChangeCount++;
          if (noChangeCount === _this.globals.orientationTestCount) end();
        } else {
          _this.data.width = currWidth;
          _this.data.height = currHeight;

          _this.resize();

          noChangeCount = 0;
        }
      }, this.globals.throttle);
      var timeout = setTimeout(function () {
        end();
      }, this.globals.orientationChangeTimeout);
    }
  }]);

  return Adaptive;
}();

export default Adaptive;
